# frozen_string_literal: true

module Wayfarer
  module Frontiers
    # @abstract The common behaviour of all frontiers.
    # @api private
    class Frontier
      attr_reader :config

      def initialize(config)
        @config = config
      end

      # Returns URIs to be scraped in the current cycle.
      # @note Usually an expensive operation!
      # @return [Array<URI>]
      def current_uris
        raise "Unimplemented"
      end

      # Returns staged URIs.
      # @return [Array<URI>]
      def staged_uris
        raise "Unimplemented"
      end

      # Stages URIs for processing in the next cycle.
      # @param [*Array<URI>, *Array<String>] uris
      def stage(*_uris)
        raise "Unimplemented"
      end

      # Whether a URI is cached.
      def staged?(_uri)
        raise "Unimplemented"
      end

      # Caches URIs so they don't get processed again.
      # @param [*Array<URI>, *Array<String>] uris
      def cache(*_uris)
        raise "Unimplemented"
      end

      # Whether a URI is cached.
      def cached?(_uri)
        raise "Unimplemented"
      end

      # Frees resources.
      def free; end

      # TODO: Documentation
      def cycle
        unless config.allow_circulation
          cache(*current_uris) # TODO: Make it a template method
          filter_staged_uris!
        end

        return false if staged_uris.none?

        swap!
        reset_staged_uris!

        true
      end

      protected

      # TODO: Documentation
      def filter_staged_uris!; end

      # TODO: Documentation
      def swap!; end

      # TODO: Documentation
      def reset_staged_uris!; end
    end
  end
end
