# frozen_string_literal: true

require "trie"

module Wayfarer
  module Frontiers
    # An in-memory trie.
    # @api private
    class MemoryTrieFrontier < MemoryFrontier
      def initialize(config)
        @trie = Trie.new
        super(config)
      end

      # @override
      def cache(*uris)
        uris.each { |uri| @trie.add(uri.to_s) }
      end

      # @override
      def match!(uri)
        @str_or_regexp === uri.host
      end

      def cached?(uri)
        # RuboCop autocorrects `#has_key?` to `#key?` otherwise
        # rubocop:disable Style/PreferredHashMethods
        @trie.has_key?(uri.to_s)
        # rubocop:enable Style/PreferredHashMethods
      end

      # @override
      def free
        @trie = nil
        super
      end
    end
  end
end
