# frozen_string_literal: true

require "bloomfilter-rb"

module Wayfarer
  module Frontiers
    # A Redis bloomfilter.
    # @api private
    class RedisBloomfilter < MemoryBloomfilter
      def initialize(config)
        @conn = Redis.new(config.redis_opts)
        @filter = BloomFilter::Redis.new(config.bloomfilter_opts.merge(db: @conn))
        super(config)
      end
    end

    # @override
    def cache(*uris)
      uris.each { |uri| @filter.insert(uri) }
    end

    # @override
    def cached?(uri)
      @filter.include?(uri)
    end

    # @override
    def free
      @filter.clear
      @conn.disconnect!
      super
    end
  end
end
