# frozen_string_literal: true

require "concurrent"

module Wayfarer
  # @api private
  module Locals
    def self.thread_safe_counterpart(value)
      case value
      when Array      then Concurrent::Array.new(value)
      when Hash       then Concurrent::Hash[value]
      when TrueClass  then Concurrent::AtomicBoolean.new(value)
      when FalseClass then Concurrent::AtomicBoolean.new(value)
      when Integer then Concurrent::AtomicFixnum.new(value)
      else value
      end
    end

    def self.included(base)
      base.extend(ClassMethods)
    end

    module ClassMethods
      attr_reader :locals

      def let(key)
        raise "#let called without a block" unless block_given?
        locals[key] = yield
      end

      def locals
        @locals ||= {}
      end

      def locals=(locals)
        @locals = locals
      end
    end
  end
end
