# frozen_string_literal: true

require "mustermann"

module Wayfarer
  module Routing
    # @private
    class PathRule < Rule
      attr_reader :matcher

      def initialize(arg, opts = {}, &proc)
        @matcher = if arg.is_a? String
                     Mustermann.new(arg, type: Wayfarer.config.mustermann_type)
                   else
                     arg
                   end

        super(opts, &proc)
      end

      def params(uri)
        return {} unless match!(uri)

        path = uri.path

        if @matcher.is_a? Mustermann
          @matcher.params(path)
        else
          captures = @matcher.match(full_path(uri)).captures

          captures.each.with_index.reduce({}) do |hash, (capture, i)|
            hash.merge(i.to_s => capture)
          end
        end
      end

      private

      # rubocop:disable Style/CaseEquality
      def match!(uri)
        if @matcher.is_a? Mustermann
          @matcher === uri.path
        else
          @matcher =~ full_path(uri)
        end
      end
      # rubocop:enable Style/CaseEquality

      def full_path(uri)
        "#{uri.path}?#{uri.query}##{uri.fragment}"
      end
    end
  end
end
